from contextvars import ContextVar
from uuid import uuid4

REQUEST_ID_CTX_KEY = "request-id"
_request_id_ctx_var: ContextVar[str] = ContextVar(REQUEST_ID_CTX_KEY, default=None)


def get_request_id() -> str:
    return _request_id_ctx_var.get()


class RequestIDMiddleware:
    """ Request ID middleware for ASGI applications
    Args:
      app (ASGI application): ASGI application
      incoming_request_id_header (string): Optional incoming request ID header
      prefix (string): Optional prefix for self generated request IDs
    """

    def __init__(self, app, incoming_request_id_header=None, prefix=None):
        if incoming_request_id_header is None:
            incoming_request_id_header = "request-id"
        if prefix is None:
            prefix = ""
        self.app = app
        self.incoming_id_header = incoming_request_id_header.lower()
        self.prefix = prefix

    async def __call__(self, scope, receive, send):
        if scope["type"] == "http":
            headers = self.get_headers(scope)
            request_id = headers.get(self.incoming_id_header)
            if not request_id:
                request_id = self.prefix + str(uuid4())
            _request_id_ctx_var.set(request_id)

        def send_wrapper(response):
            response_headers = response.get("headers")
            if response_headers:
                response["headers"].append((b"request-id", request_id.encode()))
            return send(response)

        await self.app(scope, receive, send_wrapper)

    def get_headers(self, scope):
        headers = {}
        for raw_key, raw_value in scope["headers"]:
            key = raw_key.decode("latin-1").lower()
            value = raw_value.decode("latin-1")
            if key in headers:
                headers[key] = headers[key] + ", " + value
            else:
                headers[key] = value
        return headers



import logging
import uvicorn

from starlette.applications import Starlette
from starlette.responses import PlainTextResponse

from middleware import RequestIDMiddleware, get_request_id

logger = logging.getLogger("gunicorn.error")
app = Starlette()


@app.route("/")
def homepage(request):
    logger.info(f"Request ID: {get_request_id()}")
    return PlainTextResponse("hello world")


app.add_middleware(
    RequestIDMiddleware,
    prefix="mooo-"
)

if __name__ == "__main__":
    uvicorn.run(app)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['asgi_request_id']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'asgi-request-id',
    'version': '0.1.0',
    'description': 'ASGI request id middleware',
    'long_description': 'asgi-request-id\n===============\n\nThis was developed at [GRID](https://github.com/GRID-is) for use with our\npython backend services and intended to make it easier to log/generate \nrequest IDs.\n\ninstallation\n------------\n```\npip install asgi-request-id\n```\n\nusage\n-----\n```python\nimport logging\nimport uvicorn\n\nfrom starlette.applications import Starlette\nfrom starlette.responses import PlainTextResponse\n\nfrom asgi_request_id import RequestIDMiddleware, get_request_id\n\nlogger = logging.getLogger(__name__)\napp = Starlette()\n\n\n@app.route("/")\ndef homepage(request):\n    logger.info(f"Request ID: {get_request_id()}")\n    return PlainTextResponse("hello world")\n\n\napp.add_middleware(\n    RequestIDMiddleware,\n    incoming_request_id_header="x-amzn-trace-id",\n    prefix="myapp-",\n)\n\nif __name__ == "__main__":\n    uvicorn.run(app)\n```\nThe package will do the following:\n\nSearch for an incoming request identifier and use it as the request id if found.\nIf it is not found, an unique request id with an optional prefix is generated.\n\nThe request id is stored in a context variable and made available via \n`get_request_id`\n\nFinally, it is set as the `request_id` response header.',
    'author': 'Arni Inaba Kjartansson',
    'author_email': 'arni@inaba.is',
    'url': 'https://github.com/arni-inaba/asgi-request-id',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
